"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AwsClientStub = void 0;
const sinon_1 = require("sinon");
/**
 * Wrapper on the mocked `Client#send()` method,
 * allowing to configure it's behavior.
 *
 * Without any configuration, `Client#send()` invocation returns `undefined`.
 */
class AwsClientStub {
    constructor(send) {
        this.commandMatcher = sinon_1.match.any;
        this.inputMatcher = sinon_1.match.any;
        this.send = send;
    }
    /**
     * Resets stub's history and behavior.
     */
    reset() {
        this.send.reset();
        return this;
    }
    /**
     * Resets stub's behavior.
     */
    resetBehavior() {
        this.send.resetBehavior();
        return this;
    }
    /**
     * Resets stub's calls history.
     */
    resetHistory() {
        this.send.resetHistory();
        return this;
    }
    /**
     * Replaces stub with original `Client#send()` method.
     */
    restore() {
        this.send.restore();
    }
    /**
     * Returns recorded calls to the stub.
     * Clear history with {@link resetHistory} or {@link reset}.
     * @return Array of calls
     */
    calls() {
        return this.send.getCalls();
    }
    /**
     * Returns n-th recorded call to the stub.
     * @return Call
     */
    call(n) {
        return this.send.getCall(n);
    }
    /**
     * Specifies a Command type and its input (parameters) for which the next defined `Client#send()` response
     * (with {@link resolves} or {@link rejects}) will act.
     *
     * If payload is not specified, it will match any Command of this type.
     * @param command Command type to match
     * @param input Command payload to (strictly) match
     */
    on(command, input) {
        this.resetMatchers();
        this.commandMatcher = sinon_1.match.instanceOf(command);
        if (input !== undefined) {
            this.inputMatcher = sinon_1.match.has('input', input);
        }
        return this;
    }
    /**
     * Specifies a Command input (parameters) for which the next defined `Client#send()` response
     * (with {@link resolves} or {@link rejects}) will act.
     *
     * If input is not specified, the next defined `Client#send()` response will act for any Command and any input.
     * This is only for readability, as the result is the same as with not calling it at all.
     * @param input Command payload to (strictly) match
     */
    onAnyCommand(input) {
        this.resetMatchers();
        this.inputMatcher = input !== undefined ? sinon_1.match.has('input', input) : sinon_1.match.any;
        return this;
    }
    /**
     * Sets a successful response that will be returned from the `Client#send()` invocation.
     *
     * If a Command and/or its input were specified (with {@link on} or {@link onAnyCommand}),
     * the response will be returned only on receiving them as `Client#send()` argument.
     * @param obj Content to be returned from `Client#send()` method
     */
    resolves(obj) {
        this.prepareStub().resolves(obj);
        this.resetMatchers();
        return this;
    }
    /**
     * Sets a failure response that will be returned from the `Client#send()` invocation.
     * The response will always be an `Error` instance.
     *
     * The same Command and its input rules apply as in the {@link resolves}.
     * @param error Error text, Error instance or Error parameters to be returned from `Client#send()` method
     */
    rejects(error) {
        if (typeof error === 'string') {
            error = new Error(error);
        }
        if (!(error instanceof Error)) {
            error = Object.assign(new Error(), error);
        }
        this.prepareStub().rejects(error);
        this.resetMatchers();
        return this;
    }
    /**
     * Sets a function that will be called on `Client#send()` invocation.
     *
     * The same Command and its input rules apply as in the {@link resolves}.
     * @param fn Function taking Command input and returning result
     */
    callsFake(fn) {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        this.prepareStub().callsFake(cmd => fn(cmd.input));
        this.resetMatchers();
        return this;
    }
    prepareStub() {
        return this.send.withArgs(this.getMatcher());
    }
    getMatcher() {
        return sinon_1.match.any
            .and(this.commandMatcher)
            .and(this.inputMatcher);
    }
    resetMatchers() {
        this.commandMatcher = sinon_1.match.any;
        this.inputMatcher = sinon_1.match.any;
    }
}
exports.AwsClientStub = AwsClientStub;
//# sourceMappingURL=awsClientStub.js.map