import { match } from 'sinon';
/**
 * Wrapper on the mocked `Client#send()` method,
 * allowing to configure it's behavior.
 *
 * Without any configuration, `Client#send()` invocation returns `undefined`.
 */
var AwsClientStub = /** @class */ (function () {
    function AwsClientStub(send) {
        this.commandMatcher = match.any;
        this.inputMatcher = match.any;
        this.send = send;
    }
    /**
     * Resets stub's history and behavior.
     */
    AwsClientStub.prototype.reset = function () {
        this.send.reset();
        return this;
    };
    /**
     * Resets stub's behavior.
     */
    AwsClientStub.prototype.resetBehavior = function () {
        this.send.resetBehavior();
        return this;
    };
    /**
     * Resets stub's calls history.
     */
    AwsClientStub.prototype.resetHistory = function () {
        this.send.resetHistory();
        return this;
    };
    /**
     * Replaces stub with original `Client#send()` method.
     */
    AwsClientStub.prototype.restore = function () {
        this.send.restore();
    };
    /**
     * Returns recorded calls to the stub.
     * Clear history with {@link resetHistory} or {@link reset}.
     * @return Array of calls
     */
    AwsClientStub.prototype.calls = function () {
        return this.send.getCalls();
    };
    /**
     * Returns n-th recorded call to the stub.
     * @return Call
     */
    AwsClientStub.prototype.call = function (n) {
        return this.send.getCall(n);
    };
    /**
     * Specifies a Command type and its input (parameters) for which the next defined `Client#send()` response
     * (with {@link resolves} or {@link rejects}) will act.
     *
     * If payload is not specified, it will match any Command of this type.
     * @param command Command type to match
     * @param input Command payload to (strictly) match
     */
    AwsClientStub.prototype.on = function (command, input) {
        this.resetMatchers();
        this.commandMatcher = match.instanceOf(command);
        if (input !== undefined) {
            this.inputMatcher = match.has('input', input);
        }
        return this;
    };
    /**
     * Specifies a Command input (parameters) for which the next defined `Client#send()` response
     * (with {@link resolves} or {@link rejects}) will act.
     *
     * If input is not specified, the next defined `Client#send()` response will act for any Command and any input.
     * This is only for readability, as the result is the same as with not calling it at all.
     * @param input Command payload to (strictly) match
     */
    AwsClientStub.prototype.onAnyCommand = function (input) {
        this.resetMatchers();
        this.inputMatcher = input !== undefined ? match.has('input', input) : match.any;
        return this;
    };
    /**
     * Sets a successful response that will be returned from the `Client#send()` invocation.
     *
     * If a Command and/or its input were specified (with {@link on} or {@link onAnyCommand}),
     * the response will be returned only on receiving them as `Client#send()` argument.
     * @param obj Content to be returned from `Client#send()` method
     */
    AwsClientStub.prototype.resolves = function (obj) {
        this.prepareStub().resolves(obj);
        this.resetMatchers();
        return this;
    };
    /**
     * Sets a failure response that will be returned from the `Client#send()` invocation.
     * The response will always be an `Error` instance.
     *
     * The same Command and its input rules apply as in the {@link resolves}.
     * @param error Error text, Error instance or Error parameters to be returned from `Client#send()` method
     */
    AwsClientStub.prototype.rejects = function (error) {
        if (typeof error === 'string') {
            error = new Error(error);
        }
        if (!(error instanceof Error)) {
            error = Object.assign(new Error(), error);
        }
        this.prepareStub().rejects(error);
        this.resetMatchers();
        return this;
    };
    /**
     * Sets a function that will be called on `Client#send()` invocation.
     *
     * The same Command and its input rules apply as in the {@link resolves}.
     * @param fn Function taking Command input and returning result
     */
    AwsClientStub.prototype.callsFake = function (fn) {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        this.prepareStub().callsFake(function (cmd) { return fn(cmd.input); });
        this.resetMatchers();
        return this;
    };
    AwsClientStub.prototype.prepareStub = function () {
        return this.send.withArgs(this.getMatcher());
    };
    AwsClientStub.prototype.getMatcher = function () {
        return match.any
            .and(this.commandMatcher)
            .and(this.inputMatcher);
    };
    AwsClientStub.prototype.resetMatchers = function () {
        this.commandMatcher = match.any;
        this.inputMatcher = match.any;
    };
    return AwsClientStub;
}());
export { AwsClientStub };
//# sourceMappingURL=awsClientStub.js.map